package oclib

import (
	"encoding/json"
	"errors"

	"cloud.o-forge.io/core/oc-lib/models/peer"
	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/resources/datacenter"
	"cloud.o-forge.io/core/oc-lib/models/resources/workflow/graph"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type AbstractWorkflow struct {
	resources.ResourceSet
	Graph          *graph.Graph      `bson:"graph,omitempty" json:"graph,omitempty"`
	ScheduleActive bool              `json:"schedule_active" bson:"schedule_active"`
	Schedule       *WorkflowSchedule `bson:"schedule,omitempty" json:"schedule,omitempty"`
	Shared         []string          `json:"shared,omitempty" bson:"shared,omitempty"`
}

func (w *AbstractWorkflow) isDCLink(link graph.GraphLink) (bool, string) {
	if d, ok := w.Graph.Items[link.Source.ID]; ok && d.Datacenter != nil {
		return true, d.Datacenter.UUID
	}
	if d, ok := w.Graph.Items[link.Destination.ID]; ok && d.Datacenter != nil {
		return true, d.Datacenter.UUID
	}
	return false, ""
}

type Workflow struct {
	utils.AbstractObject
	AbstractWorkflow
}

func (wfa *Workflow) CheckBooking(caller *tools.HTTPCaller) (bool, error) {
	// check if
	if wfa.Graph == nil {
		return false, nil
	}
	accessor := (&datacenter.DatacenterResource{}).GetAccessor(nil)
	for _, link := range wfa.Graph.Links {
		if ok, dc_id := wfa.isDCLink(link); ok {
			dc, code, _ := accessor.LoadOne(dc_id)
			if code != 200 {
				continue
			}
			// CHECK BOOKING
			peerID := dc.(*datacenter.DatacenterResource).PeerID
			if peerID == "" {
				return false, errors.New("no peer id")
			}
			_, err := (&peer.Peer{}).LaunchPeerExecution(peerID, dc_id, utils.BOOKING, tools.GET, nil, caller)
			if err != nil {
				return false, err
			}
		}
	}
	return true, nil
}

func (d *Workflow) GetName() string {
	return d.Name
}

func (d *Workflow) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := New()
	data.Init(utils.WORKFLOW, caller)
	return data
}

func (dma *Workflow) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *Workflow) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
